@@ -41,7 +41,7 @@ from sphinx_gallery.utils import (
 #
 # total number of plot_*.py files in
 # (examples + examples_rst_index + examples_with_rst + examples_README_header)
-N_EXAMPLES = 17 + 3 + 2 + 1
+N_EXAMPLES = 19 + 3 + 2 + 1
 N_FAILING = 4
 N_GOOD = N_EXAMPLES - N_FAILING  # galleries that run w/o error
 # passthroughs and non-executed examples in
@@ -54,7 +54,7 @@ N_INDEX = 2 + 1 + 3 + 1
 # (examples + examples_rst_index + examples_with_rst + examples_README_header + root-level)
 N_EXECUTE = 2 + 3 + 1 + 1 + 1
 # gen_modules + sg_api_usage + doc/index.rst + minigallery.rst
-N_OTHER = 9 + 1 + 1 + 1 + 1
+N_OTHER = 11 + 1 + 1 + 1 + 1
 N_RST = N_EXAMPLES + N_PASS + N_INDEX + N_EXECUTE + N_OTHER
 N_RST = f"({N_RST}|{N_RST - 1}|{N_RST - 2})"  # AppVeyor weirdness
 
@@ -83,7 +83,14 @@ def _sphinx_app(tmpdir_factory, buildername):
     src_dir = Path(__file__).parent / "tinybuild"
 
     def ignore(src, names):
-        return ("_build", "gen_modules", "auto_examples")
+        return (
+            "_build",
+            "gen_modules",
+            "auto_examples",
+            "auto_examples_README_header",
+            "auto_examples_rst_index",
+            "auto_examples_with_rst",
+        )
 
     shutil.copytree(src_dir, temp_dir, ignore=ignore)
     # For testing iteration, you can get similar behavior just doing `make`
@@ -308,6 +315,13 @@ def test_run_sphinx(sphinx_app):
     assert re.match(want, warning, re.DOTALL) is not None, warning
 
 
+def test_user_index_download(sphinx_app):
+    """Test download zipfiles still generated when user supplies index.rst"""
+    src_dir = Path(sphinx_app.srcdir) / "auto_examples_rst_index"
+    assert (src_dir / "auto_examples_rst_index_jupyter.zip").is_file()
+    assert (src_dir / "auto_examples_rst_index_python.zip").is_file()
+
+
 def test_thumbnail_path(sphinx_app, tmpdir):
     """Test sphinx_gallery_thumbnail_path."""
     import numpy as np
@@ -395,6 +409,36 @@ def test_thumbnail_expected_failing_examples(sphinx_app, tmpdir):
     assert corr < 0.7  # i.e. thumbnail and "BROKEN" stamp are not identical
 
 
+def test_multi_image(sphinx_app):
+    """Test `sphinx_gallery_multi_image(_block)` variables."""
+    generated_examples_dir = op.join(sphinx_app.outdir, "auto_examples")
+
+    # Check file-wide `sphinx_gallery_multi_image="single"` produces no multi-img
+    html_fname = op.join(generated_examples_dir, "plot_multi_image_separate.html")
+    with codecs.open(html_fname, "r", "utf-8") as fid:
+        html = fid.read()
+    assert "sphx-glr-single-img" in html
+    assert "sphx-glr-multi-img" not in html
+
+    # Check block-specific `sphinx_gallery_multi_image_block` produces mixed img classes
+    html_fname = op.join(generated_examples_dir, "plot_multi_image_block_separate.html")
+    with codecs.open(html_fname, "r", "utf-8") as fid:
+        html = fid.read()
+    # find start of each code block
+    matches = re.finditer('<div class="highlight-Python notranslate">', html)
+    starts = [match.start() for match in matches] + [-1]
+    assert len(starts) == 4  # 3 code block plus an extra for end index
+    for block_idx, (start, end) in enumerate(zip(starts[:-1], starts[1:])):
+        # ignore first code block (just imports)
+        block_html = html[start:end]
+        if block_idx == 1:  # multi-img classes for this code block
+            assert "sphx-glr-multi-img" in block_html
+            assert "sphx-glr-single-img" not in block_html
+        elif block_idx == 2:  # single-img classes for this code block
+            assert "sphx-glr-single-img" in block_html
+            assert "sphx-glr-multi-img" not in block_html
+
+
 def test_command_line_args_img(sphinx_app):
     generated_examples_dir = op.join(sphinx_app.outdir, "auto_examples")
     thumb_fname = "../_images/sphx_glr_plot_command_line_args_thumb.png"
@@ -534,25 +578,49 @@ def test_embed_links_and_styles(sphinx_app):
     # issue 617 (regex '-'s)
     # instance
     dummy_class_inst = re.search(
-        r'sphinx_gallery.backreferences.html#sphinx[_,-]gallery[.,-]backreferences[.,-][D,d]ummy[C,c]lass" title="sphinx_gallery.backreferences.DummyClass" class="sphx-glr-backref-module-sphinx_gallery-backreferences sphx-glr-backref-type-py-class sphx-glr-backref-instance"><span class="n">dc</span>',  # noqa: E501
+        r'sphinx_gallery._dummy.html#sphinx[_-]gallery[.-]_dummy[.-][Dd]ummy[Cc]lass" title="sphinx_gallery._dummy.DummyClass" class="sphx-glr-backref-module-sphinx_gallery-_dummy sphx-glr-backref-type-py-class sphx-glr-backref-instance"><span class="n">dc</span>',  # noqa: E501
         lines,
     )
     assert dummy_class_inst is not None
     # class
     dummy_class_class = re.search(
-        r'sphinx_gallery.backreferences.html#sphinx[_,-]gallery[.,-]backreferences[.,-][D,d]ummy[C,c]lass" title="sphinx_gallery.backreferences.DummyClass" class="sphx-glr-backref-module-sphinx_gallery-backreferences sphx-glr-backref-type-py-class"><span class="n">sphinx_gallery</span><span class="o">.</span><span class="n">backreferences</span><span class="o">.</span><span class="n">DummyClass</span>',  # noqa: E501
+        r'sphinx_gallery._dummy.html#sphinx[_-]gallery[.-]_dummy[.-][Dd]ummy[Cc]lass" title="sphinx_gallery._dummy.DummyClass" class="sphx-glr-backref-module-sphinx_gallery-_dummy sphx-glr-backref-type-py-class"><span class="n">sphinx_gallery</span><span class="o">.</span><span class="n">_dummy</span><span class="o">.</span><span class="n">DummyClass</span>',  # noqa: E501
         lines,
     )
     assert dummy_class_class is not None
     # method
     dummy_class_meth = re.search(
-        r'sphinx_gallery.backreferences.html#sphinx[_,-]gallery[.,-]backreferences[.,-][D,d]ummy[C,c]lass[.,-]run" title="sphinx_gallery.backreferences.DummyClass.run" class="sphx-glr-backref-module-sphinx_gallery-backreferences sphx-glr-backref-type-py-method"><span class="n">dc</span><span class="o">.</span><span class="n">run</span>',  # noqa: E501
+        r'sphinx_gallery._dummy.html#sphinx[_-]gallery[.-]_dummy[.-][Dd]ummy[Cc]lass[.-]run" title="sphinx_gallery._dummy.DummyClass.run" class="sphx-glr-backref-module-sphinx_gallery-_dummy sphx-glr-backref-type-py-method"><span class="n">dc</span><span class="o">.</span><span class="n">run</span>',  # noqa: E501
         lines,
     )
     assert dummy_class_meth is not None
     # property (Sphinx 2+ calls it a method rather than attribute, so regex)
     dummy_class_prop = re.compile(
-        r'sphinx_gallery.backreferences.html#sphinx[_,-]gallery[.,-]backreferences[.,-][D,d]ummy[C,c]lass[.,-]prop" title="sphinx_gallery.backreferences.DummyClass.prop" class="sphx-glr-backref-module-sphinx_gallery-backreferences sphx-glr-backref-type-py-(attribute|method|property)"><span class="n">dc</span><span class="o">.</span><span class="n">prop</span>'
+        r'sphinx_gallery._dummy.html#sphinx[_-]gallery[.-]_dummy[.-][Dd]ummy[Cc]lass[.-]prop" title="sphinx_gallery._dummy.DummyClass.prop" class="sphx-glr-backref-module-sphinx_gallery-_dummy sphx-glr-backref-type-py-(attribute|method|property)"><span class="n">dc</span><span class="o">.</span><span class="n">prop</span>'
+    )  # noqa: E501
+    assert dummy_class_prop.search(lines) is not None
+    # gh-1364: methods of nested classes in the module currently being documented
+    # instance
+    dummy_class_inst = re.search(
+        r'sphinx_gallery._dummy.nested.html#sphinx[_-]gallery[.-]_dummy[.-]nested[.-][Nn]ested[Dd]ummy[Cc]lass" title="sphinx_gallery._dummy.NestedDummyClass" class="sphx-glr-backref-module-sphinx_gallery-_dummy sphx-glr-backref-type-py-class sphx-glr-backref-instance"><span class="n">ndc</span>',  # noqa: E501
+        lines,
+    )
+    assert dummy_class_inst is not None
+    # class
+    dummy_class_class = re.search(
+        r'sphinx_gallery._dummy.nested.html#sphinx[_-]gallery[.-]_dummy[.-]nested[.-][Nn]ested[Dd]ummy[Cc]lass" title="sphinx_gallery._dummy.NestedDummyClass" class="sphx-glr-backref-module-sphinx_gallery-_dummy sphx-glr-backref-type-py-class"><span class="n">sphinx_gallery</span><span class="o">.</span><span class="n">_dummy</span><span class="o">.</span><span class="n">nested</span><span class="o">.</span><span class="n">NestedDummyClass</span>',  # noqa: E501
+        lines,
+    )
+    assert dummy_class_class is not None
+    # method
+    dummy_class_meth = re.search(
+        r'sphinx_gallery._dummy.nested.html#sphinx[_-]gallery[.-]_dummy[.-]nested[.-][Nn]ested[Dd]ummy[Cc]lass[.-]run" title="sphinx_gallery._dummy.NestedDummyClass.run" class="sphx-glr-backref-module-sphinx_gallery-_dummy sphx-glr-backref-type-py-method"><span class="n">ndc</span><span class="o">.</span><span class="n">run</span>',  # noqa: E501
+        lines,
+    )
+    assert dummy_class_meth is not None
+    # property (Sphinx 2+ calls it a method rather than attribute, so regex)
+    dummy_class_prop = re.compile(
+        r'sphinx_gallery._dummy.nested.html#sphinx[_-]gallery[.-]_dummy[.-]nested[.-][Nn]ested[Dd]ummy[Cc]lass[.-]prop" title="sphinx_gallery._dummy.NestedDummyClass.prop" class="sphx-glr-backref-module-sphinx_gallery-_dummy sphx-glr-backref-type-py-(attribute|method|property)"><span class="n">ndc</span><span class="o">.</span><span class="n">prop</span>'
     )  # noqa: E501
     assert dummy_class_prop.search(lines) is not None
 
@@ -652,19 +720,70 @@ def test_backreferences_examples_html(sphinx_app):
     regex = re.compile(r'<dt[ \S]*id="sphinx_gallery.backreferences.[ \S]*>')
     n_documented = len(regex.findall(lines))
     possible = "\n".join(line for line in lines.split("\n") if "<dt " in line)
-    # identify_names, DummyClass, DummyClass.prop, DummyClass.run, NameFinder
-    # NameFinder.get_mapping, NameFinder.visit_Attribute, NameFinder.visit_Import
-    # NameFinder.visit_ImportFrom, NameFinder.visit_Name
-    assert n_documented == 10, possible
-    # identify_names, DummyClass, NameFinder (3); once doc, once left bar (x2)
+    # identify_names, NameFinder NameFinder.get_mapping, NameFinder.visit_Attribute,
+    # NameFinder.visit_Import NameFinder.visit_ImportFrom, NameFinder.visit_Name
+    assert n_documented == 7, possible
+    # identify_names, NameFinder (2); once doc, once left bar (x2)
     n_mini = lines.count("Examples using ")
-    assert n_mini == 6
-    # only 3 actual mini-gallery divs
+    assert n_mini == 4
+    # only 2 actual mini-gallery divs
     n_div = lines.count('<div class="sphx-glr-thumbnails')
-    assert n_div == 3
+    assert n_div == 2
+    # 2 documented uses
+    n_thumb = lines.count('<div class="sphx-glr-thumbcontainer')
+    assert n_thumb == 2
+    # matched opening/closing divs
+    n_open = lines.count("<div")
+    n_close = lines.count("</div")
+    assert n_open == n_close  # should always be equal
+
+    backref_file = op.join(
+        sphinx_app.outdir, "gen_modules", "sphinx_gallery._dummy.html"
+    )
+    with codecs.open(backref_file, "r", "utf-8") as fid:
+        lines = fid.read()
+    # Class properties not properly checked on older Sphinx (e.g. 3)
+    # so let's use the "id" instead
+    regex = re.compile(r'<dt[ \S]*id="sphinx_gallery._dummy.[ \S]*>')
+    n_documented = len(regex.findall(lines))
+    possible = "\n".join(line for line in lines.split("\n") if "<dt " in line)
+    # DummyClass, DummyClass.prop, DummyClass.run
+    assert n_documented == 3, possible
+    # DummyClass (1); once doc, once left bar (x2)
+    n_mini = lines.count("Examples using ")
+    assert n_mini == 2
+    # only 1 actual mini-gallery divs
+    n_div = lines.count('<div class="sphx-glr-thumbnails')
+    assert n_div == 1
     # 3 documented uses
     n_thumb = lines.count('<div class="sphx-glr-thumbcontainer')
-    assert n_thumb == 3
+    assert n_thumb == 1
+    # matched opening/closing divs
+    n_open = lines.count("<div")
+    n_close = lines.count("</div")
+    assert n_open == n_close  # should always be equal
+
+    backref_file = op.join(
+        sphinx_app.outdir, "gen_modules", "sphinx_gallery._dummy.nested.html"
+    )
+    with codecs.open(backref_file, "r", "utf-8") as fid:
+        lines = fid.read()
+    # Class properties not properly checked on older Sphinx (e.g. 3)
+    # so let's use the "id" instead
+    regex = re.compile(r'<dt[ \S]*id="sphinx_gallery._dummy.nested.[ \S]*>')
+    n_documented = len(regex.findall(lines))
+    possible = "\n".join(line for line in lines.split("\n") if "<dt " in line)
+    # NestedDummyClass, NestedDummyClass.prop, NestedDummyClass.run
+    assert n_documented == 3, possible
+    # DummyClass (1); once doc, once left bar (x2)
+    n_mini = lines.count("Examples using ")
+    assert n_mini == 2
+    # only 1 actual mini-gallery divs
+    n_div = lines.count('<div class="sphx-glr-thumbnails')
+    assert n_div == 1
+    # 3 documented uses
+    n_thumb = lines.count('<div class="sphx-glr-thumbcontainer')
+    assert n_thumb == 1
     # matched opening/closing divs
     n_open = lines.count("<div")
     n_close = lines.count("</div")
@@ -800,7 +919,7 @@ def test_rebuild(tmpdir_factory, sphinx_app):
     else:
         assert (
             re.match(
-                ".*[0|1] added, ([1-9]|10) changed, 0 removed$.*",
+                ".*[0|1] added, ([1-9]|1[0-1]) changed, 0 removed$.*",
                 status,
                 re.MULTILINE | re.DOTALL,
             )
@@ -1453,9 +1572,9 @@ def test_recommend_n_examples(sphinx_app):
     assert '<p class="rubric">Related examples</p>' in html
     assert count == n_examples
     # Check the same 3 related examples are shown (can change when new examples added)
-    assert "sphx-glr-auto-examples-plot-repr-py" in html
-    assert "sphx-glr-auto-examples-plot-matplotlib-backend-py" in html
-    assert "sphx-glr-auto-examples-plot-second-future-imports-py" in html
+    assert "sphx-glr-auto-examples-plot-defer-figures-py" in html
+    assert "sphx-glr-auto-examples-plot-webp-py" in html
+    assert "sphx-glr-auto-examples-plot-command-line-args-py" in html
 
 
 def test_sidebar_components_download_links(sphinx_app):
