@@ -2,12 +2,14 @@
 # License: 3-clause BSD
 r"""Test Sphinx-Gallery gallery generation."""
 
+import inspect
 import json
 import os
 import re
 from pathlib import Path
 
 import pytest
+from sphinx.application import Sphinx
 from sphinx.config import is_serializable
 from sphinx.errors import ConfigError, ExtensionError, SphinxWarning
 
@@ -80,7 +82,7 @@ def test_serializable(sphinx_app_wrapper):
     assert not bad, f"Non-serializable values found:\n{bad}"
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -100,6 +102,20 @@ def test_no_warning_simple_config(sphinx_app_wrapper):
     assert build_warn == ""
 
 
+@pytest.mark.add_conf(
+    content="""
+sphinx_gallery_conf = {
+    'examples_dirs': ['src', 'excess'],
+    'gallery_dirs': 'ex',
+}"""
+)
+def test_unequal_examples_gallery_dirs(sphinx_app_wrapper):
+    """Check warning when 'examples_dirs' and 'gallery_dirs' unequal lengths."""
+    sphinx_app = sphinx_app_wrapper.create_sphinx_app()
+    build_warn = sphinx_app._warning.getvalue()
+    assert "'examples_dirs' and 'gallery_dirs' are of different lengths" in build_warn
+
+
 # This changed from passing the ValueError directly to
 # raising "sphinx.errors.ConfigError" with "threw an exception"
 @pytest.mark.parametrize(
@@ -109,7 +125,7 @@ def test_no_warning_simple_config(sphinx_app_wrapper):
             ConfigError,
             "Unknown string option for reset_modules",
             id="Resetter unknown",
-            marks=pytest.mark.conf_file(
+            marks=pytest.mark.add_conf(
                 content="sphinx_gallery_conf={'reset_modules': ('f',)}"
             ),
         ),
@@ -117,7 +133,7 @@ def test_no_warning_simple_config(sphinx_app_wrapper):
             ConfigError,
             "reset_modules.* must be callable",
             id="Resetter not callable",
-            marks=pytest.mark.conf_file(
+            marks=pytest.mark.add_conf(
                 content="sphinx_gallery_conf={'reset_modules': (1.,),}"
             ),
         ),
@@ -135,7 +151,7 @@ def test_bad_reset(sphinx_app_wrapper, err_class, err_match):
             ConfigError,
             "'reset_modules_order' config allowed types",
             id="Resetter unknown",
-            marks=pytest.mark.conf_file(
+            marks=pytest.mark.add_conf(
                 content=("sphinx_gallery_conf=" "{'reset_modules_order': 1,}")
             ),
         ),
@@ -143,7 +159,7 @@ def test_bad_reset(sphinx_app_wrapper, err_class, err_match):
             ConfigError,
             "reset_modules_order must be in",
             id="reset_modules_order not valid",
-            marks=pytest.mark.conf_file(
+            marks=pytest.mark.add_conf(
                 content=("sphinx_gallery_conf=" "{'reset_modules_order': 'invalid',}")
             ),
         ),
@@ -161,15 +177,13 @@ def test_bad_reset_modules_order(sphinx_app_wrapper, err_class, err_match):
             ConfigError,
             "Unknown css",
             id="CSS str error",
-            marks=pytest.mark.conf_file(
-                content="sphinx_gallery_conf={'css': ('foo',)}"
-            ),
+            marks=pytest.mark.add_conf(content="sphinx_gallery_conf={'css': ('foo',)}"),
         ),
         pytest.param(
             ConfigError,
             "config allowed types:",
             id="CSS type error",
-            marks=pytest.mark.conf_file(content="sphinx_gallery_conf={'css': 1.}"),
+            marks=pytest.mark.add_conf(content="sphinx_gallery_conf={'css': 1.}"),
         ),
     ],
 )
@@ -189,7 +203,7 @@ def test_bad_api():
         _fill_gallery_conf_defaults(sphinx_gallery_conf)
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'backreferences_dir': os.path.join('gen_modules', 'backreferences'),
@@ -259,7 +273,7 @@ def _check_order(sphinx_app, key):
 
     Used to test that these keys (in index.rst) appear in a specific order.
     """
-    index_fname = os.path.join(sphinx_app.outdir, "..", "ex", "index.rst")
+    index_fname = Path(sphinx_app.outdir, "..", "ex", "index.rst")
     order = list()
     regex = f".*:{key}=(.):.*"
     with open(index_fname, "r", encoding="utf-8") as fid:
@@ -270,7 +284,7 @@ def _check_order(sphinx_app, key):
     assert order == [1, 2, 3]
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -283,7 +297,7 @@ def test_example_sorting_default(sphinx_app_wrapper):
     _check_order(sphinx_app, "lines")
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -297,7 +311,7 @@ def test_example_sorting_filesize(sphinx_app_wrapper):
     _check_order(sphinx_app, "filesize")
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -311,7 +325,7 @@ def test_example_sorting_filename(sphinx_app_wrapper):
     _check_order(sphinx_app, "filename")
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -387,13 +401,28 @@ def test_collect_gallery_files_ignore_pattern(tmpdir, gallery_conf):
     expected_files = {
         ap.strpath
         for ap in abs_paths
-        if re.search(r"one", os.path.basename(ap.strpath)) is None
+        if re.search(r"one", Path(ap.strpath).name) is None
     }
 
     assert collected_files == expected_files
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
+    content=r"""
+sphinx_gallery_conf = {
+    'examples_dirs': 'src',
+    'gallery_dirs': 'ex',
+    'copyfile_regex': r'.*\.rst',
+}"""
+)
+@pytest.mark.add_rst(file="own index.rst")
+def test_own_index_first(sphinx_app_wrapper):
+    """Test `generate_gallery_rst` works when own index gallery is first (and only)."""
+    # Issue #1382
+    sphinx_app_wrapper.build_sphinx_app()
+
+
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'backreferences_dir' : os.path.join('modules', 'gen'),
@@ -412,22 +441,20 @@ def test_binder_copy_files(sphinx_app_wrapper):
     sphinx_app = sphinx_app_wrapper.create_sphinx_app()
     gallery_conf = sphinx_app.config.sphinx_gallery_conf
     # Create requirements file
-    with open(os.path.join(sphinx_app.srcdir, "requirements.txt"), "w"):
+    with open(Path(sphinx_app.srcdir, "requirements.txt"), "w"):
         pass
     copy_binder_files(sphinx_app, None)
 
     for i_file in ["plot_1", "plot_2", "plot_3"]:
-        assert os.path.exists(
-            os.path.join(
-                sphinx_app.outdir,
-                "ntbk_folder",
-                gallery_conf["gallery_dirs"][0],
-                i_file + ".ipynb",
-            )
-        )
+        assert Path(
+            sphinx_app.outdir,
+            "ntbk_folder",
+            gallery_conf["gallery_dirs"][0],
+            i_file + ".ipynb",
+        ).exists()
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -435,7 +462,7 @@ sphinx_gallery_conf = {
 }"""
 )
 def test_failing_examples_raise_exception(sphinx_app_wrapper):
-    example_dir = os.path.join(sphinx_app_wrapper.srcdir, "src")
+    example_dir = Path(sphinx_app_wrapper.srcdir, "src")
     bad_line = "print(f'{a[}')"  # never closed bracket inside print -> SyntaxError
     bad_code = f"""\
 '''
@@ -451,7 +478,7 @@ Should emit a syntax error in the second code block.
 {bad_line}
 """
     bad_line_no = bad_code.split("\n").index(bad_line) + 1
-    with open(os.path.join(example_dir, "plot_3.py"), "w", encoding="utf-8") as fid:
+    with open(Path(example_dir, "plot_3.py"), "w", encoding="utf-8") as fid:
         fid.write(bad_code)
     with pytest.raises(ExtensionError) as excinfo:
         sphinx_app_wrapper.build_sphinx_app()
@@ -462,7 +489,7 @@ Should emit a syntax error in the second code block.
     assert bad_line in tb
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -478,7 +505,7 @@ def test_expected_failing_examples_were_executed(sphinx_app_wrapper):
     sphinx_app_wrapper.build_sphinx_app()
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -498,7 +525,7 @@ def test_only_warn_on_example_error(sphinx_app_wrapper):
     assert "WARNING: Here is a summary of the problems" in build_warn
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -508,7 +535,10 @@ sphinx_gallery_conf = {
 )
 def test_only_warn_on_example_error_sphinx_warning(sphinx_app_wrapper):
     """Test behaviour of only_warn_on_example_error flag."""
-    sphinx_app_wrapper.kwargs["warningiserror"] = True
+    # https://github.com/sphinx-doc/sphinx/pull/12743/files
+    for key in ("warningiserror", "exception_on_warning"):
+        if key in inspect.getfullargspec(Sphinx).args:
+            sphinx_app_wrapper.kwargs[key] = True
     example_dir = Path(sphinx_app_wrapper.srcdir) / "src"
     with open(example_dir / "plot_3.py", "w", encoding="utf-8") as fid:
         fid.write(f"{MINIMAL_HEADER}raise ValueError")
@@ -518,7 +548,7 @@ def test_only_warn_on_example_error_sphinx_warning(sphinx_app_wrapper):
     assert "plot_3.py unexpectedly failed to execute" in exc
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -533,12 +563,13 @@ def test_examples_not_expected_to_pass(sphinx_app_wrapper):
     assert "expected to fail, but not failing" in exc
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 from sphinx_gallery.gen_rst import _sg_call_memory_noop
 
 sphinx_gallery_conf = {
     'show_memory': _sg_call_memory_noop,
+    'examples_dirs': 'src',
     'gallery_dirs': 'ex',
 }"""
 )
@@ -553,13 +584,13 @@ def test_show_memory_callable(sphinx_app_wrapper):
     [
         pytest.param(
             id="first notebook cell",
-            marks=pytest.mark.conf_file(
+            marks=pytest.mark.add_conf(
                 content="""sphinx_gallery_conf = {'first_notebook_cell': 2,}"""
             ),
         ),
         pytest.param(
             id="last notebook cell",
-            marks=pytest.mark.conf_file(
+            marks=pytest.mark.add_conf(
                 content="""sphinx_gallery_conf = {'last_notebook_cell': 2,}"""
             ),
         ),
@@ -572,7 +603,7 @@ def test_notebook_cell_config(sphinx_app_wrapper):
         fill_gallery_conf_defaults(app, app.config, check_keys=False)
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'backreferences_dir': False,
@@ -585,7 +616,21 @@ def test_backreferences_dir_config(sphinx_app_wrapper):
         fill_gallery_conf_defaults(app, app.config, check_keys=False)
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
+    content="""
+sphinx_gallery_conf = {
+    'examples_dirs': 'src',
+    'gallery_dirs': 'ex',
+}"""
+)
+def test_minigallery_no_backreferences_dir(sphinx_app_wrapper):
+    """Check warning when no backreferences_dir set but minigallery directive used."""
+    sphinx_app = sphinx_app_wrapper.build_sphinx_app()
+    build_warn = sphinx_app._warning.getvalue()
+    assert "'backreferences_dir' config is None, minigallery" in build_warn
+
+
+@pytest.mark.add_conf(
     content="""
 import pathlib
 
@@ -599,6 +644,59 @@ def test_backreferences_dir_pathlib_config(sphinx_app_wrapper):
     fill_gallery_conf_defaults(app, app.config, check_keys=False)
 
 
+@pytest.mark.add_conf(
+    content="""
+sphinx_gallery_conf = {
+    'examples_dirs': 'src',
+    'gallery_dirs': 'ex',
+}"""
+)
+@pytest.mark.add_rst(
+    file="""
+Header
+======
+
+.. minigallery:: index.rst
+"""
+)
+def test_minigallery_not_in_examples_dirs(sphinx_app_wrapper):
+    """Check error when minigallery directive's path input not in `examples_dirs`."""
+    msg = "minigallery directive error: path input 'index.rst'"
+    with pytest.raises(ExtensionError, match=msg):
+        sphinx_app_wrapper.build_sphinx_app()
+
+
+@pytest.mark.add_conf(
+    content="""
+sphinx_gallery_conf = {
+    'examples_dirs': ['src', 'src/sub_folder/sub_sub_folder'],
+    'gallery_dirs': ['ex', 'ex/sub_folder/sub_sub_folder'],
+}"""
+)
+@pytest.mark.add_rst(
+    file="""
+Header
+======
+
+.. minigallery:: src/sub_folder/sub_sub_folder/plot_nested.py
+"""
+)
+def test_minigallery_multi_match(sphinx_app_wrapper):
+    """Check minigallery directive's path input resolution in nested `examples_dirs`.
+
+    When a examples gallery is nested inside another examples gallery, path inputs
+    from the nested gallery should resolve to the nested gallery.
+    """
+    sphinx_app = sphinx_app_wrapper.build_sphinx_app()
+    minigallery_html = Path(sphinx_app.outdir) / "minigallery_test.html"
+    with open(minigallery_html, "r") as fid:
+        mg_html = fid.read()
+    # Check thumbnail correct
+    assert "_images/sphx_glr_plot_nested_thumb.png" in mg_html
+    # Check href correct
+    assert "sphx-glr-ex-sub-folder-sub-sub-folder-plot-nested-py" in mg_html
+
+
 def test_write_computation_times_noop(sphinx_app_wrapper):
     app = sphinx_app_wrapper.create_sphinx_app()
     write_computation_times(app.config.sphinx_gallery_conf, None, [])
@@ -608,7 +706,7 @@ def test_write_api_usage_noop(sphinx_app_wrapper):
     write_api_entry_usage(sphinx_app_wrapper.create_sphinx_app(), list(), None)
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'pypandoc': ['list',],
@@ -624,7 +722,7 @@ def test_pypandoc_config_list(sphinx_app_wrapper):
         fill_gallery_conf_defaults(app, app.config, check_keys=False)
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'pypandoc': {'bad key': 1},
@@ -639,7 +737,7 @@ def test_pypandoc_config_keys(sphinx_app_wrapper):
         fill_gallery_conf_defaults(app, app.config, check_keys=False)
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 extensions += ['jupyterlite_sphinx']
 
@@ -658,17 +756,15 @@ def test_create_jupyterlite_contents(sphinx_app_wrapper):
     create_jupyterlite_contents(sphinx_app, exception=None)
 
     for i_file in ["plot_1", "plot_2", "plot_3"]:
-        assert os.path.exists(
-            os.path.join(
-                sphinx_app.srcdir,
-                "jupyterlite_contents",
-                gallery_conf["gallery_dirs"][0],
-                i_file + ".ipynb",
-            )
-        )
+        assert Path(
+            sphinx_app.srcdir,
+            "jupyterlite_contents",
+            gallery_conf["gallery_dirs"][0],
+            i_file + ".ipynb",
+        ).exists()
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 extensions += ['jupyterlite_sphinx']
 
@@ -688,17 +784,15 @@ def test_create_jupyterlite_contents_non_default_contents(sphinx_app_wrapper):
     create_jupyterlite_contents(sphinx_app, exception=None)
 
     for i_file in ["plot_1", "plot_2", "plot_3"]:
-        assert os.path.exists(
-            os.path.join(
-                sphinx_app.srcdir,
-                "this_is_the_contents_dir",
-                gallery_conf["gallery_dirs"][0],
-                i_file + ".ipynb",
-            )
-        )
+        assert Path(
+            sphinx_app.srcdir,
+            "this_is_the_contents_dir",
+            gallery_conf["gallery_dirs"][0],
+            i_file + ".ipynb",
+        ).exists()
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'backreferences_dir' : os.path.join('modules', 'gen'),
@@ -714,10 +808,10 @@ def test_create_jupyterlite_contents_without_jupyterlite_sphinx_loaded(
     sphinx_app = sphinx_app_wrapper.create_sphinx_app()
 
     create_jupyterlite_contents(sphinx_app, exception=None)
-    assert not os.path.exists(os.path.join(sphinx_app.srcdir, "jupyterlite_contents"))
+    assert not Path(sphinx_app.srcdir, "jupyterlite_contents").exists()
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 extensions += ['jupyterlite_sphinx']
 
@@ -739,10 +833,10 @@ def test_create_jupyterlite_contents_with_jupyterlite_disabled_via_config(
     sphinx_app = sphinx_app_wrapper.create_sphinx_app()
 
     create_jupyterlite_contents(sphinx_app, exception=None)
-    assert not os.path.exists(os.path.join(sphinx_app.outdir, "jupyterlite_contents"))
+    assert not Path(sphinx_app.outdir, "jupyterlite_contents").exists()
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 extensions += ['jupyterlite_sphinx']
 
@@ -773,13 +867,13 @@ def test_create_jupyterlite_contents_with_modification(sphinx_app_wrapper):
     create_jupyterlite_contents(sphinx_app, exception=None)
 
     for i_file in ["plot_1", "plot_2", "plot_3"]:
-        notebook_filename = os.path.join(
+        notebook_filename = Path(
             sphinx_app.srcdir,
             "jupyterlite_contents",
             gallery_conf["gallery_dirs"][0],
             i_file + ".ipynb",
         )
-        assert os.path.exists(notebook_filename)
+        assert notebook_filename.exists()
 
         with open(notebook_filename) as f:
             notebook_content = json.load(f)
